#include <LiquidCrystal_I2C.h>
#include <SoftwareSerial.h>

//Create software serial object to communicate with SIM800L
SoftwareSerial sim(3, 2); //SIM800L Tx & Rx is connected to Arduino #3 & #2

LiquidCrystal_I2C lcd(0x3F, 16, 2);

// defines pins numbers
const int trigPin = 9;
const int echoPin = 10;
const int buzzer = 11;
const int ledPin = 13;
const int delayMessage = 200;
const int delayCall = 200000;
const int buttonPin = 4;
const int ALARM_DIST = 4;
const int MESSAGE_DIST = 4;

// defines variables
long duration;
int distance;
int safetyDistance;
int messageSent = 0;
int timeAfterSent = 0;

void stopButton() {
  noTone(buzzer);
}

void sendMessage() {
  sim.println("AT+CMGF=1"); // Configuring TEXT mode
  updateSerial();
  sim.println("AT+CMGS=\"+40769558771\"");
  updateSerial();
  sim.print("Movement detected!"); //text content
  updateSerial();
  sim.write(26);
}

void alarmCall() {
  tone(buzzer, 1000);

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Alarm is on!");
  lcd.setCursor(1,1);
  lcd.print("Calling..");

  sim.println("ATD+ +40769558771;"); 
  updateSerial();
  sim.write(26);
  delay(delayCall);
  sim.println("ATH");
  updateSerial();
}

void setup() {
    // Starts the serial communication
  Serial.begin(9600); 
  //Begin serial communication with Arduino and SIM800L
  sim.begin(9600);

  Serial.println("Initializing..."); 
  delay(1000);

  sim.println("AT");
  updateSerial();

  // Button interruption
  pinMode(buttonPin, INPUT_PULLUP);
  attachInterrupt(buttonPin, stopButton, RISING);
 
  // Initializing display
  lcd.init();
  lcd.backlight();

  // Set pins
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  pinMode(echoPin, INPUT); // Sets the echoPin as an Input
  pinMode(buzzer, OUTPUT);
  pinMode(ledPin, OUTPUT);
}

void loop() {
  // Clears the trigPin
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);

  // Check if message was sent and if freeze time for message is gone
  if (timeAfterSent < delayMessage && messageSent == 1) {
    lcd.setCursor(1, 1);
    lcd.print("Message sent!");
    timeAfterSent++;
  } else {
    timeAfterSent = 0;
    messageSent = 0;
  }

  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);

  // Calculating the distance
  distance = duration * 0.034 / 2;

  safetyDistance = distance;

  // Prints the distance on the Serial Monitor
  lcd.setCursor(0, 0);
  lcd.print("Distance:");
  lcd.print(distance);

  lcd.setCursor(1, 1);
  // Check distance
  if (distance <= ALARM_DIST) {
    alarmCall();
  } else if (distance <= MESSAGE_DIST && messageSent == 0) {
    messageSent = 1;
    sendMessage();
  } else {
    noTone(buzzer);
  }

  delay(100);
  lcd.clear();
}

void updateSerial()
{
  delay(500);
  while (Serial.available()) 
  {
    sim.write(Serial.read());
  }
  while(sim.available()) 
  {
    Serial.write(sim.read());
  }
}
